package com.qmth.distributed.print.api;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.bean.result.TSyncExamStudentScoreResult;
import com.qmth.distributed.print.business.entity.TBSyncTask;
import com.qmth.distributed.print.business.entity.TSyncExamStudentScore;
import com.qmth.distributed.print.business.enums.ImageTrajectoryEnum;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.distributed.print.business.service.TSyncExamStudentScoreService;
import com.qmth.distributed.print.business.templete.execute.AsyncScoreBatchDownloadService;
import com.qmth.distributed.print.business.templete.execute.AsyncScoreExportService;
import com.qmth.distributed.print.business.templete.execute.AsyncScorePushService;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PushTypeEnum;
import com.qmth.teachcloud.common.enums.TaskTypeEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.util.FileUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.apache.commons.io.IOUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 同步考生成绩表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2021-10-28
 */
@Api(tags = "成绩归档Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sync}")
@Aac(auth = BOOL.FALSE, strict = BOOL.FALSE)
@Validated
public class TSyncExamStudentScoreController {

    @Resource
    TSyncExamStudentScoreService tSyncExamStudentScoreService;

    @Resource
    PrintCommonService printCommonService;

    @Resource
    AsyncScoreExportService asyncScoreExportService;

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    AsyncScorePushService asyncScorePushService;

    @Resource
    AsyncScoreBatchDownloadService asyncScoreBatchDownloadService;

    @ApiOperation(value = "成绩归档查询列表")
    @ApiResponses({@ApiResponse(code = 200, message = "成绩查询信息", response = TSyncExamStudentScoreResult.class)})
    @RequestMapping(value = "/score/list", method = RequestMethod.POST)
    public Result list(@ApiParam(value = "学期id", required = true) @RequestParam String semesterId,
                       @ApiParam(value = "学院id", required = true) @RequestParam String orgId,
                       @ApiParam(value = "专业id", required = false) @RequestParam(required = false) String majorId,
                       @ApiParam(value = "班级id", required = false) @RequestParam(required = false) String clazzId,
                       @ApiParam(value = "课程编码", required = false) @RequestParam(required = false) String courseCode,
                       @ApiParam(value = "页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @ApiParam(value = "数量", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(tSyncExamStudentScoreService.list(new Page<>(pageNumber, pageSize),
                SystemConstant.convertIdToLong(semesterId),
                SystemConstant.convertIdToLong(orgId),
                SystemConstant.convertIdToLong(majorId),
                SystemConstant.convertIdToLong(clazzId),
                courseCode));
    }

    @ApiOperation(value = "成绩导出")
    @ApiResponses({@ApiResponse(code = 200, message = "同步信息", response = TBTask.class)})
    @RequestMapping(value = "/score/export", method = RequestMethod.POST)
    public Result export(@ApiParam(value = "学期id", required = true) @RequestParam String semesterId,
                         @ApiParam(value = "学院id", required = true) @RequestParam String orgId,
                         @ApiParam(value = "专业id", required = false) @RequestParam(required = false) String majorId,
                         @ApiParam(value = "班级id", required = false) @RequestParam(required = false) String clazzId,
                         @ApiParam(value = "课程编码", required = false) @RequestParam(required = false) String courseCode) {
        Map<String, Object> map = printCommonService.saveTask(TaskTypeEnum.SCORE_EXPORT);
        map.computeIfAbsent("semesterId", v -> SystemConstant.convertIdToLong(semesterId));
        map.computeIfAbsent("orgId", v -> SystemConstant.convertIdToLong(orgId));
        map.computeIfAbsent("majorId", v -> SystemConstant.convertIdToLong(majorId));
        map.computeIfAbsent("clazzId", v -> SystemConstant.convertIdToLong(clazzId));
        map.computeIfAbsent("courseCode", v -> courseCode);
        map.computeIfAbsent(SystemConstant.OSS, v -> dictionaryConfig.sysDomain().isOss());
        asyncScoreExportService.exportTask(map);
        TBTask tbTask = Objects.nonNull(map.get(SystemConstant.TASK)) ? (TBTask) map.get(SystemConstant.TASK) : null;
        return Objects.nonNull(tbTask) ? ResultUtil.ok(tbTask.getId()) : ResultUtil.error("创建任务失败");
    }

    @ApiOperation(value = "成绩查询同步")
    @ApiResponses({@ApiResponse(code = 200, message = "同步异步任务信息", response = TBSyncTask.class)})
    @RequestMapping(value = "/score/sync", method = RequestMethod.POST)
    public Result sync(@ApiParam(value = "考试id", required = true) @RequestParam String examId,
                       @ApiParam(value = "考试编码", required = false) @RequestParam(required = false) String examCode) {
        Map<String, Object> map = printCommonService.savePush(PushTypeEnum.SCORE_PUSH);
        map.computeIfAbsent("examId", v -> SystemConstant.convertIdToInteger(examId));
        map.computeIfAbsent("examCode", v -> examCode);
        asyncScorePushService.pushTask(map);
        TBSyncTask tbSyncTask = Objects.nonNull(map.get(SystemConstant.TB_SYNC_TASK)) ? (TBSyncTask) map.get(SystemConstant.TB_SYNC_TASK) : null;
        return Objects.nonNull(tbSyncTask) ? ResultUtil.ok(tbSyncTask.getId()) : ResultUtil.error("创建同步推送任务失败");
    }

    @CrossOrigin(maxAge = 3600) //支持跨域
    @ApiOperation(value = "成绩动态轨迹图下载")
    @ApiResponses({@ApiResponse(code = 200, message = "常规信息", response = EditResult.class)})
    @RequestMapping(value = "/score/download", method = RequestMethod.POST)
    @Transactional
    public void download(@ApiParam(value = "学号", required = true) @RequestParam String studentCode) throws Exception {
//        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        QueryWrapper<TSyncExamStudentScore> tSyncExamStudentScoreQueryWrapper = new QueryWrapper<>();
        tSyncExamStudentScoreQueryWrapper.lambda().eq(TSyncExamStudentScore::getSchoolId, 2L).eq(TSyncExamStudentScore::getStudentCode, studentCode);
        TSyncExamStudentScore tSyncExamStudentScore = tSyncExamStudentScoreService.getOne(tSyncExamStudentScoreQueryWrapper);
        Optional.ofNullable(tSyncExamStudentScore).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未找到此考生信息"));
        boolean update = Objects.isNull(tSyncExamStudentScore.getTrajectoryUrls()) ? true : false;
        tSyncExamStudentScore = tSyncExamStudentScoreService.createImageTrajectory(tSyncExamStudentScore, ImageTrajectoryEnum.PREVIEW);
        Optional.ofNullable(tSyncExamStudentScore.getTrajectoryUrls()).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("资源未获取到，请稍候再试"));
        if (update || Objects.isNull(tSyncExamStudentScore.getTrajectoryUrls())) {
            tSyncExamStudentScoreService.updateById(tSyncExamStudentScore);
        }
        File zipFile = null;
        OutputStream outputStream = null;
        try {
            if (Objects.nonNull(tSyncExamStudentScore.getTrajectoryFileList())) {
                StringJoiner stringJoiner = new StringJoiner("");
                stringJoiner.add(SystemConstant.TEMP_FILES_DIR).add(File.separator);
                LocalDateTime nowTime = LocalDateTime.now();
                StringJoiner dirZipName = new StringJoiner("");
                dirZipName.add(UploadFileEnum.FILE.getTitle()).add(File.separator)
                        .add(String.valueOf(nowTime.getYear())).add(File.separator)
                        .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                        .add(String.format("%02d", nowTime.getDayOfMonth()))
                        .add(File.separator).add(SystemConstant.getUuid()).add(SystemConstant.ZIP_PREFIX);
                zipFile = new File(stringJoiner.toString() + dirZipName.toString());
                if (!zipFile.exists()) {
                    zipFile.getParentFile().mkdirs();
                    zipFile.createNewFile();
                }
                FileUtil.doZip(zipFile, Arrays.asList(tSyncExamStudentScore.getTrajectoryFileList().get(0).getParentFile()));
                HttpServletResponse response = ServletUtil.getResponse();
//                response.setContentType("image/jpeg");
//                outputStream = new BufferedOutputStream(response.getOutputStream());
//                IOUtils.copy(new FileInputStream(zipFile), outputStream);
                response.reset();
                response.setHeader("Content-Disposition", "attachment;fileName=" + zipFile.getName());
                response.addHeader("Content-Length", "" + zipFile.length());
                response.setContentType("application/octet-stream;charset=UTF-8");
                outputStream = new BufferedOutputStream(response.getOutputStream());
                IOUtils.copy(new FileInputStream(zipFile), outputStream);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(outputStream)) {
                outputStream.flush();
                outputStream.close();
            }
            if (Objects.nonNull(zipFile)) {
                zipFile.delete();
            }
        }
    }

    @ApiOperation(value = "成绩动态轨迹图一键下载")
    @ApiResponses({@ApiResponse(code = 200, message = "同步信息", response = TBTask.class)})
    @RequestMapping(value = "/score/batch_download", method = RequestMethod.POST)
    public Result batchDownload(@ApiParam(value = "学期id", required = true) @RequestParam String semesterId,
                                @ApiParam(value = "学院id", required = true) @RequestParam String orgId,
                                @ApiParam(value = "专业id", required = false) @RequestParam(required = false) String majorId,
                                @ApiParam(value = "班级id", required = false) @RequestParam(required = false) String clazzId,
                                @ApiParam(value = "课程编码", required = false) @RequestParam(required = false) String courseCode) {
        Map<String, Object> map = printCommonService.saveTask(TaskTypeEnum.SCORE_DOWNLOAD);
        map.computeIfAbsent("semesterId", v -> SystemConstant.convertIdToLong(semesterId));
        map.computeIfAbsent("orgId", v -> SystemConstant.convertIdToLong(orgId));
        map.computeIfAbsent("majorId", v -> SystemConstant.convertIdToLong(majorId));
        map.computeIfAbsent("clazzId", v -> SystemConstant.convertIdToLong(clazzId));
        map.computeIfAbsent("courseCode", v -> courseCode);
        map.computeIfAbsent(SystemConstant.OSS, v -> dictionaryConfig.sysDomain().isOss());
        asyncScoreBatchDownloadService.exportTask(map);
        TBTask tbTask = Objects.nonNull(map.get(SystemConstant.TASK)) ? (TBTask) map.get(SystemConstant.TASK) : null;
        return Objects.nonNull(tbTask) ? ResultUtil.ok(tbTask.getId()) : ResultUtil.error("创建任务失败");
    }
}
